==> go-test-framework-223.8836.34-sources.jar: com/goide/vgo/VgoTestConfigurator.java <==
// Copyright 2016-2021 JetBrains s.r.o.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package com.goide.vgo;

import com.goide.vgo.configuration.VgoProjectSettings;
import com.goide.vgo.mod.psi.VgoExcludeDirective;
import com.goide.vgo.mod.psi.VgoFile;
import com.goide.vgo.mod.psi.VgoReplaceDirective;
import com.goide.vgo.mod.psi.VgoRequireDirective;
import com.goide.vgo.mod.psi.impl.VgoElementFactory;
import com.goide.vgo.project.*;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import com.intellij.openapi.editor.Document;
import com.intellij.openapi.util.io.FileUtil;
import com.intellij.openapi.util.text.StringUtil;
import com.intellij.openapi.vfs.VfsUtil;
import com.intellij.openapi.vfs.VfsUtilCore;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.psi.PsiDocumentManager;
import com.intellij.psi.PsiElement;
import com.intellij.testFramework.EditorTestUtil;
import com.intellij.testFramework.ExpectedHighlightingData;
import com.intellij.testFramework.PlatformTestUtil;
import com.intellij.testFramework.fixtures.BasePlatformTestCase;
import com.intellij.testFramework.fixtures.CodeInsightTestFixture;
import com.intellij.util.SmartList;
import com.intellij.util.containers.ContainerUtil;
import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Configures Go Module tests using files in testData to fully describe the module and its dependencies.
 * <p>
 * testData for your test should have 3 components: <br/>
 * 1. Source code of the module, i.e. go.mod and *.go files <br/>
 * 2. Source code of the module's dependencies <br/>
 * 3. dependencies.json file that connects the module with the dependencies by specifying the locations of the latter. <br/>
 * <p>
 * Example:
 * <p>
 * Suppose you need to write a test for an inspection called 'VgoBadCodeInspection'. Here is how you can do that: <br/>
 * 1. Create testData/inspections/badCode/ directory <br/>
 * 2. Put go.mod and *.go files needed for the test in testData/inspections/badCode/ <br/>
 * 3. Put dependencies in testData/inspection/badCode_gopath/pkg/mod <br/>
 * 4. Create testData/inspection/badCode/dependencies.json and describe dependencies there (can be omitted, see details below). <br/>
 * 5. In VgoBadCodeInspectionTest call <br/>
 * new VgoTestConfigurator(myFixture, "badCode").setModules("badCode").setGopath("badCode_gopath").configure();
 * <p>
 * How to create dependencies.json:
 * <p>
 * If you run the IDE, create your module there and execute "go list -m -json all" from the terminal,
 * you will get the contents of the dependencies.json (you need to adjust it slightly, "go list" outputs separate json objects,
 * but in dependencies.json you store them as a json array).
 * <p>
 * By the way, this enables a workflow when you test your changes in the IDE,
 * then just copy all the files and the "go list" output to create Java tests.
 * <p>
 * Notes:
 * <p>
 * 1. You can omit dependencies.json. In that case, configurator will try to infer dependencies from the go.mod. <br/>
 * 2. You can use a global gopath defined for all tests in testData/mockGoPath. Just don't call .setGopath(). <br/>
 * 3. You can use $GOMODCACHE variable in dependencies.json to refer to testData/inspection/badCode_gopath/pkg/mod. <br/>
 * 4. You can use $TESTDATA variable in dependencies.json to refer to testData/inspection/badCode. <br/>
 * <p>
 * See {@link VgoDocumentationTest} as an example. <br/>
 * See {@link VgoTestUtil#setupVgoIntegration(CodeInsightTestFixture)} if you don't need a custom Go Module, just Go Modules integration.
 */
public class VgoTestConfigurator {
  private @NotNull final CodeInsightTestFixture myFixture;
  private @NotNull final String myTestDataPath;
  private @Nullable List<String> myModulePaths;
  private @Nullable List<String> myWorkspacePaths;
  private @Nullable String myCustomGopath;

  /**
   * @param testDataPath most of the time this is {@link BasePlatformTestCase#getTestName(boolean)}
   */
  public VgoTestConfigurator(@NotNull CodeInsightTestFixture fixture, @NotNull String testDataPath) {
    myFixture = fixture;
    myTestDataPath = testDataPath;
  }

  /**
   * If you omit this method call, test will use {@code testDataPath} passed to the constructor as a module path.
   *
   * @param paths most of the time this is {@link BasePlatformTestCase#getTestName(boolean)}, but if you have nested modules, it could be
   *              {@code setModules(testName, testName + "/nestedModule"})
   */
  @Contract(pure = true)
  public VgoTestConfigurator setModules(String @NotNull ... paths) {
    myModulePaths = ContainerUtil.newArrayList(paths);
    return this;
  }

  @Contract(pure = true)
  public VgoTestConfigurator setWorkspaces(String @NotNull ... paths) {
    myWorkspacePaths = ContainerUtil.newArrayList(paths);
    return this;
  }

  /**
   * If you omit this method call, test will use testData/mockGoPath.
   */
  @Contract(pure = true)
  public VgoTestConfigurator setGopath(@NotNull String gopath) {
    myCustomGopath = gopath;
    return this;
  }

  public @NotNull List<VgoModule> configure() {
    if (myModulePaths == null) {
      myModulePaths = List.of();
    }
    myFixture.copyDirectoryToProject(myTestDataPath, myTestDataPath);
    VirtualFile gopath = VgoTestUtil.setupGopath(myFixture, myCustomGopath);
    List<VgoModule> vgoModules = new SmartList<>();
    if (myWorkspacePaths != null) {
      vgoModules.addAll(ContainerUtil.flatMap(myWorkspacePaths,
                                              workspacePath -> readWorkspace(myFixture, myTestDataPath, workspacePath, gopath)));
    }
    else if (myModulePaths.isEmpty()) {
      myModulePaths = Collections.singletonList(myTestDataPath);
    }
    vgoModules.addAll(ContainerUtil.map(myModulePaths, modulePath -> readModule(myFixture, myTestDataPath, modulePath, gopath)));
    if (VgoProjectSettings.getInstance(myFixture.getProject()).isIntegrationEnabled()) {
      VgoTestUtil.setVgoModules(myFixture, vgoModules);
      PlatformTestUtil.dispatchAllInvocationEventsInIdeEventQueue();
    }
    else {
      VgoTestUtil.setupVgoIntegration(myFixture, vgoModules);
    }
    return vgoModules;
  }

  @NotNull
  private static List<VgoModule> readWorkspace(@NotNull CodeInsightTestFixture fixture,
                                               @NotNull String testDataPath,
                                               @NotNull String workspacePath,
                                               @NotNull VirtualFile gopath) {
    VirtualFile goWorkFile = fixture.findFileInTempDir(workspacePath + "/" + VgoUtil.GO_WORK);
    if (goWorkFile == null) {
      throw new IllegalStateException("Cannot find go.work file in test data: " + fixture.getTempDirPath() + "/" + workspacePath);
    }
    VirtualFile dependenciesJson = fixture.getTempDirFixture().getFile(workspacePath + "/dependencies.json");
    if (dependenciesJson == null) {
      throw new IllegalStateException("Workspace mode requires dependencies.json");
    }
    GoListOutputParser.BuildList buildList = readBuildList(fixture, testDataPath, dependenciesJson, gopath);
    VgoWorkspace workspace = new VgoWorkspace(buildList, goWorkFile);
    return ContainerUtil.map(buildList.getMainModules(), mainModule -> new VgoWorkspaceModule(Objects.requireNonNull(mainModule.getRoot()),
                                                                                              mainModule.getImportPath(),
                                                                                              mainModule.getGoVersion(),
                                                                                              workspace));
  }

  private static @NotNull VgoModule readModule(@NotNull CodeInsightTestFixture fixture,
                                               @NotNull String testDataPath,
                                               @NotNull String modulePath,
                                               @NotNull VirtualFile gopath) {
    VirtualFile goModFile = fixture.findFileInTempDir(modulePath + "/go.mod");
    if (goModFile == null) {
      throw new IllegalStateException("Cannot find go.mod file in test data: " + fixture.getTempDirPath() + "/" + modulePath);
    }
    VgoFile goModCopy = parseGoModFile(fixture, goModFile);
    String moduleName = Objects.requireNonNull(goModCopy.getModuleName());
    List<VgoDependency> dependencies = readDependencies(fixture, testDataPath, modulePath, gopath, goModCopy);
    return new VgoStandaloneModule(goModFile.getParent(), moduleName, goModCopy.getGoVersion(), dependencies);
  }

  @NotNull
  private static VgoFile parseGoModFile(@NotNull CodeInsightTestFixture fixture, @NotNull VirtualFile goModFile) {
    String goModText;
    try {
      goModText = VfsUtilCore.loadText(goModFile);
    }
    catch (IOException e) {
      throw new IllegalStateException("Cannot read go.mod file from test data: " + goModFile.getPath());
    }
    VgoFile goModCopy = VgoElementFactory.createFileFromText(fixture.getProject(), goModText);
    Document document = PsiDocumentManager.getInstance(fixture.getProject()).getDocument(goModCopy);
    EditorTestUtil.extractCaretAndSelectionMarkers(Objects.requireNonNull(document));
    // Drop expected highlighting markup from the document.
    ExpectedHighlightingData expectedData = new ExpectedHighlightingData(document);
    expectedData.init();
    PsiDocumentManager.getInstance(fixture.getProject()).commitDocument(document);
    return goModCopy;
  }

  private static @NotNull List<VgoDependency> readDependencies(@NotNull CodeInsightTestFixture fixture,
                                                               @NotNull String testDataPath,
                                                               @NotNull String modulePath,
                                                               @NotNull VirtualFile gopath,
                                                               @NotNull VgoFile goMod) {
    VirtualFile dependenciesJson = fixture.getTempDirFixture().getFile(modulePath + "/dependencies.json");
    return dependenciesJson == null
           ? calculateDependencies(goMod, gopath)
           : readBuildList(fixture, testDataPath, dependenciesJson, gopath).getDependencies();
  }

  @NotNull
  private static List<VgoDependency> calculateDependencies(@NotNull VgoFile goMod, @NotNull VirtualFile gopath) {
    if (goMod.findChildByClass(VgoReplaceDirective.class) != null || goMod.findChildByClass(VgoExcludeDirective.class) != null) {
      throw new IllegalStateException("Cannot calculate dependencies, the replace and exclude directives are not supported. " +
                                      "Consider adding dependencies.json.");
    }
    return Arrays.stream(goMod.findChildrenByClass(VgoRequireDirective.class))
      .flatMap(r -> r.getModuleSpecList().stream()).map(spec -> {
        String importPath = StringUtil.unquoteString(spec.getIdentifier().getText());
        PsiElement version = spec.getModuleVersion();
        if (version == null) {
          throw new IllegalStateException("Cannot calculate dependencies, found a dependency without a version. " +
                                          "Consider adding dependencies.json.");
        }
        String versionText = version.getText();
        Path dependencyPath = Paths.get(FileUtil.toSystemIndependentName(gopath.getPath()), "pkg", "mod", importPath + "@" + versionText);
        VirtualFile dependencyDir = VfsUtil.findFile(dependencyPath, true);
        String dependencyDirPath = dependencyDir != null ? dependencyDir.getPath() : null;
        return new VgoDependencyImpl(importPath, versionText, null, dependencyDirPath, null, null);
      })
      .collect(Collectors.toList());
  }

  @NotNull
  private static GoListOutputParser.BuildList readBuildList(@NotNull CodeInsightTestFixture fixture,
                                                            @NotNull String testDataPath,
                                                            @NotNull VirtualFile dependenciesJson,
                                                            @NotNull VirtualFile gopath) {
    try {
      String dependenciesText = VfsUtilCore.loadText(dependenciesJson)
        .replaceAll("\\$GOMODCACHE", FileUtil.toSystemIndependentName(gopath.getPath()) + "/pkg/mod")
        .replaceAll("\\$TESTDATA", fixture.findFileInTempDir(testDataPath).getUrl());
      Gson gson = new Gson();
      List<String> separateDependencies = ContainerUtil.map(gson.fromJson(dependenciesText, Object[].class), gson::toJson);
      return new GoListOutputParser().readDependencies(StringUtil.join(separateDependencies, "\n"));
    }
    catch (IOException | JsonSyntaxException e) {
      throw new RuntimeException(e);
    }
  }
}

==> go-test-framework-231.9161.41-sources.jar: com/goide/vgo/VgoTestConfigurator.java <==
// Copyright 2016-2021 JetBrains s.r.o.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package com.goide.vgo;

import com.goide.vgo.configuration.VgoProjectSettings;
import com.goide.vgo.mod.psi.VgoExcludeDirective;
import com.goide.vgo.mod.psi.VgoFile;
import com.goide.vgo.mod.psi.VgoReplaceDirective;
import com.goide.vgo.mod.psi.VgoRequireDirective;
import com.goide.vgo.mod.psi.impl.VgoElementFactory;
import com.goide.vgo.project.*;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import com.intellij.openapi.editor.Document;
import com.intellij.openapi.util.io.FileUtil;
import com.intellij.openapi.util.text.StringUtil;
import com.intellij.openapi.vfs.VfsUtil;
import com.intellij.openapi.vfs.VfsUtilCore;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.psi.PsiDocumentManager;
import com.intellij.psi.PsiElement;
import com.intellij.testFramework.EditorTestUtil;
import com.intellij.testFramework.ExpectedHighlightingData;
import com.intellij.testFramework.PlatformTestUtil;
import com.intellij.testFramework.fixtures.BasePlatformTestCase;
import com.intellij.testFramework.fixtures.CodeInsightTestFixture;
import com.intellij.util.SmartList;
import com.intellij.util.containers.ContainerUtil;
import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Configures Go Module tests using files in testData to fully describe the module and its dependencies.
 * <p>
 * testData for your test should have 3 components: <br/>
 * 1. Source code of the module, i.e. go.mod and *.go files <br/>
 * 2. Source code of the module's dependencies <br/>
 * 3. dependencies.json file that connects the module with the dependencies by specifying the locations of the latter. <br/>
 * <p>
 * Example:
 * <p>
 * Suppose you need to write a test for an inspection called 'VgoBadCodeInspection'. Here is how you can do that: <br/>
 * 1. Create testData/inspections/badCode/ directory <br/>
 * 2. Put go.mod and *.go files needed for the test in testData/inspections/badCode/ <br/>
 * 3. Put dependencies in testData/inspection/badCode_gopath/pkg/mod <br/>
 * 4. Create testData/inspection/badCode/dependencies.json and describe dependencies there (can be omitted, see details below). <br/>
 * 5. In VgoBadCodeInspectionTest call <br/>
 * new VgoTestConfigurator(myFixture, "badCode").setModules("badCode").setGopath("badCode_gopath").configure();
 * <p>
 * How to create dependencies.json:
 * <p>
 * If you run the IDE, create your module there and execute "go list -m -json all" from the terminal,
 * you will get the contents of the dependencies.json (you need to adjust it slightly, "go list" outputs separate json objects,
 * but in dependencies.json you store them as a json array).
 * <p>
 * By the way, this enables a workflow when you test your changes in the IDE,
 * then just copy all the files and the "go list" output to create Java tests.
 * <p>
 * Notes:
 * <p>
 * 1. You can omit dependencies.json. In that case, configurator will try to infer dependencies from the go.mod. <br/>
 * 2. You can use a global gopath defined for all tests in testData/mockGoPath. Just don't call .setGopath(). <br/>
 * 3. You can use $GOMODCACHE variable in dependencies.json to refer to testData/inspection/badCode_gopath/pkg/mod. <br/>
 * 4. You can use $TESTDATA variable in dependencies.json to refer to testData/inspection/badCode. <br/>
 * <p>
 * See {@link VgoDocumentationTest} as an example. <br/>
 * See {@link VgoTestUtil#setupVgoIntegration(CodeInsightTestFixture)} if you don't need a custom Go Module, just Go Modules integration.
 */
public class VgoTestConfigurator {
  private @NotNull final CodeInsightTestFixture myFixture;
  private @NotNull final String myTestDataPath;
  private @Nullable List<String> myModulePaths;
  private @Nullable List<String> myWorkspacePaths;
  private @Nullable String myCustomGopath;

  /**
   * @param testDataPath most of the time this is {@link BasePlatformTestCase#getTestName(boolean)}
   */
  public VgoTestConfigurator(@NotNull CodeInsightTestFixture fixture, @NotNull String testDataPath) {
    myFixture = fixture;
    myTestDataPath = testDataPath;
  }

  /**
   * If you omit this method call, test will use {@code testDataPath} passed to the constructor as a module path.
   *
   * @param paths most of the time this is {@link BasePlatformTestCase#getTestName(boolean)}, but if you have nested modules, it could be
   *              {@code setModules(testName, testName + "/nestedModule"})
   */
  @Contract(pure = true)
  public VgoTestConfigurator setModules(String @NotNull ... paths) {
    myModulePaths = List.of(paths);
    return this;
  }

  @Contract(pure = true)
  public VgoTestConfigurator setWorkspaces(String @NotNull ... paths) {
    myWorkspacePaths = List.of(paths);
    return this;
  }

  /**
   * If you omit this method call, test will use testData/mockGoPath.
   */
  @Contract(pure = true)
  public VgoTestConfigurator setGopath(@NotNull String gopath) {
    myCustomGopath = gopath;
    return this;
  }

  public @NotNull List<VgoModule> configure() {
    if (myModulePaths == null) {
      myModulePaths = List.of();
    }
    myFixture.copyDirectoryToProject(myTestDataPath, myTestDataPath);
    VirtualFile gopath = VgoTestUtil.setupGopath(myFixture, myCustomGopath);
    List<VgoModule> vgoModules = new SmartList<>();
    if (myWorkspacePaths != null) {
      vgoModules.addAll(ContainerUtil.flatMap(myWorkspacePaths,
                                              workspacePath -> readWorkspace(myFixture, myTestDataPath, workspacePath, gopath)));
    }
    else if (myModulePaths.isEmpty()) {
      myModulePaths = Collections.singletonList(myTestDataPath);
    }
    vgoModules.addAll(ContainerUtil.map(myModulePaths, modulePath -> readModule(myFixture, myTestDataPath, modulePath, gopath)));
    if (VgoProjectSettings.getInstance(myFixture.getProject()).isIntegrationEnabled()) {
      VgoTestUtil.setVgoModules(myFixture, vgoModules);
      PlatformTestUtil.dispatchAllInvocationEventsInIdeEventQueue();
    }
    else {
      VgoTestUtil.setupVgoIntegration(myFixture, vgoModules);
    }
    return vgoModules;
  }

  @NotNull
  private static List<VgoModule> readWorkspace(@NotNull CodeInsightTestFixture fixture,
                                               @NotNull String testDataPath,
                                               @NotNull String workspacePath,
                                               @NotNull VirtualFile gopath) {
    VirtualFile goWorkFile = fixture.findFileInTempDir(workspacePath + "/" + VgoUtil.GO_WORK);
    if (goWorkFile == null) {
      throw new IllegalStateException("Cannot find go.work file in test data: " + fixture.getTempDirPath() + "/" + workspacePath);
    }
    VirtualFile dependenciesJson = fixture.getTempDirFixture().getFile(workspacePath + "/dependencies.json");
    if (dependenciesJson == null) {
      throw new IllegalStateException("Workspace mode requires dependencies.json");
    }
    GoListOutputParser.BuildList buildList = readBuildList(fixture, testDataPath, dependenciesJson, gopath);
    VgoWorkspace workspace = new VgoWorkspace(buildList, goWorkFile);
    return ContainerUtil.map(buildList.getMainModules(), mainModule -> new VgoWorkspaceModule(Objects.requireNonNull(mainModule.getRoot()),
                                                                                              mainModule.getImportPath(),
                                                                                              mainModule.getGoVersion(),
                                                                                              workspace));
  }

  private static @NotNull VgoModule readModule(@NotNull CodeInsightTestFixture fixture,
                                               @NotNull String testDataPath,
                                               @NotNull String modulePath,
                                               @NotNull VirtualFile gopath) {
    VirtualFile goModFile = fixture.findFileInTempDir(modulePath + "/go.mod");
    if (goModFile == null) {
      throw new IllegalStateException("Cannot find go.mod file in test data: " + fixture.getTempDirPath() + "/" + modulePath);
    }
    VgoFile goModCopy = parseGoModFile(fixture, goModFile);
    String moduleName = Objects.requireNonNull(goModCopy.getModuleName());
    List<VgoDependency> dependencies = readDependencies(fixture, testDataPath, modulePath, gopath, goModCopy);
    return new VgoStandaloneModule(goModFile.getParent(), moduleName, goModCopy.getGoVersion(), dependencies);
  }

  @NotNull
  private static VgoFile parseGoModFile(@NotNull CodeInsightTestFixture fixture, @NotNull VirtualFile goModFile) {
    String goModText;
    try {
      goModText = VfsUtilCore.loadText(goModFile);
    }
    catch (IOException e) {
      throw new IllegalStateException("Cannot read go.mod file from test data: " + goModFile.getPath());
    }
    VgoFile goModCopy = VgoElementFactory.createFileFromText(fixture.getProject(), goModText);
    Document document = PsiDocumentManager.getInstance(fixture.getProject()).getDocument(goModCopy);
    EditorTestUtil.extractCaretAndSelectionMarkers(Objects.requireNonNull(document));
    // Drop expected highlighting markup from the document.
    ExpectedHighlightingData expectedData = new ExpectedHighlightingData(document);
    expectedData.init();
    PsiDocumentManager.getInstance(fixture.getProject()).commitDocument(document);
    return goModCopy;
  }

  private static @NotNull List<VgoDependency> readDependencies(@NotNull CodeInsightTestFixture fixture,
                                                               @NotNull String testDataPath,
                                                               @NotNull String modulePath,
                                                               @NotNull VirtualFile gopath,
                                                               @NotNull VgoFile goMod) {
    VirtualFile dependenciesJson = fixture.getTempDirFixture().getFile(modulePath + "/dependencies.json");
    return dependenciesJson == null
           ? calculateDependencies(goMod, gopath)
           : readBuildList(fixture, testDataPath, dependenciesJson, gopath).getDependencies();
  }

  @NotNull
  private static List<VgoDependency> calculateDependencies(@NotNull VgoFile goMod, @NotNull VirtualFile gopath) {
    if (goMod.findChildByClass(VgoReplaceDirective.class) != null || goMod.findChildByClass(VgoExcludeDirective.class) != null) {
      throw new IllegalStateException("Cannot calculate dependencies, the replace and exclude directives are not supported. " +
                                      "Consider adding dependencies.json.");
    }
    return Arrays.stream(goMod.findChildrenByClass(VgoRequireDirective.class))
      .flatMap(r -> r.getModuleSpecList().stream()).map(spec -> {
        String importPath = StringUtil.unquoteString(spec.getIdentifier().getText());
        PsiElement version = spec.getModuleVersion();
        if (version == null) {
          throw new IllegalStateException("Cannot calculate dependencies, found a dependency without a version. " +
                                          "Consider adding dependencies.json.");
        }
        String versionText = version.getText();
        Path dependencyPath = Paths.get(FileUtil.toSystemIndependentName(gopath.getPath()), "pkg", "mod", importPath + "@" + versionText);
        VirtualFile dependencyDir = VfsUtil.findFile(dependencyPath, true);
        String dependencyDirPath = dependencyDir != null ? dependencyDir.getPath() : null;
        return new VgoDependencyImpl(importPath, versionText, null, dependencyDirPath, null, null);
      })
      .collect(Collectors.toList());
  }

  @NotNull
  private static GoListOutputParser.BuildList readBuildList(@NotNull CodeInsightTestFixture fixture,
                                                            @NotNull String testDataPath,
                                                            @NotNull VirtualFile dependenciesJson,
                                                            @NotNull VirtualFile gopath) {
    try {
      String dependenciesText = VfsUtilCore.loadText(dependenciesJson)
        .replaceAll("\\$GOMODCACHE", FileUtil.toSystemIndependentName(gopath.getPath()) + "/pkg/mod")
        .replaceAll("\\$TESTDATA", fixture.findFileInTempDir(testDataPath).getUrl());
      Gson gson = new Gson();
      List<String> separateDependencies = ContainerUtil.map(gson.fromJson(dependenciesText, Object[].class), gson::toJson);
      return new GoListOutputParser().readDependencies(StringUtil.join(separateDependencies, "\n"));
    }
    catch (IOException | JsonSyntaxException e) {
      throw new RuntimeException(e);
    }
  }
}
